#lang slideshow

(require slideshow/code)
(require "fonctions.ss")


; ########################
;
; introduction a fluxus
;
; ########################


; on charge d'abord l'aide à l'installation
(require "slideshow1.ss")



;chapitre 1: introduction à fluxus
(chapitrage2 'firstart)

(slide
 (para "Lançons Fluxus à partir du terminal:")
 (cmd ">fluxus")
 'next
 (para "L'éditeur de Fluxus s'ouvre alors, avec un prompt dans un TOPLEVEL. Ce TOPLEVEL vous permettra de tester en temps réel les expressions que vous tapez.")
 'next
 (para "Faisons un essai, nous allons créer un cube qui serra affiché:")
 (code (build-cube)))

(slide
 (para "Vous devriez voir apparaître un cube derrière le TOPLEVEL.")
 'next
 (para "Vous pouvez changer l'angle de vue ou zommer avec la souris.")
 'next
 (para "Fluxus met à votre disposition 9 espaces de travail, accessibles avec Ctrl-[0-9].")
 (blank)
 (it "Ctrl-0 est le TOPLEVEL"))

(slide
 (para/center "Il existe plusieurs primitives de base dans Fluxus. On vient de voir build-cube qui produit un cube générique. On peut de la même manière créer des balles, des plans, des polygones et même des donuts !"))

(slide
 #:title "Le rendu"
 (para  "Il existe deux façons de concevoir le \"dessin\" à l'écran:")
 'next
 (item "à la volée")
 (subitem "on va expliciter l'affichage à l'écran.")
 'next
 (item "avec le scene graph")
 (subitem "utilisation de la pile de la scène."))

(slide
 #:title "Le rendu à la volée"
 (para "au TOPLEVEL, tapez:")
 (code (clear))
 (blank)
 (blank)
 (it "Vous verrez plus loin ce que fait exactement cette commande."))

(slide
 #:title "Le rendu à la volée"
 (para "Déplaçons nous dans le premier espace de travail avec Ctrl-1 puis tapons notre premier script:")
 (code (every-frame (draw-cube)))
 'next
 (para "Puis lançons le avec Ctrl-e ou F5."))

(slide
 #:title "Le rendu à la volée"
 (para/center "La fonction every-frame est appelée à chaque redessinage de l'écran; elle prend en paramètre une fonction qu'elle va executer à chaque fois que le rendu se fait. Ici nous avons explicité l'affichage d'un cube.")
 'next
 (para/center "L'autre manière est d'utiliser la pile de rendu de la scène."))

(slide
 #:title "Le rendu par la pile de scène"
 #:layout 'top
 (para "L'autre manière est d'utiliser la pile de la scène. Nous ne dessinons pas vraiment à l'écran, mais nous créons des objets, ces objets sont ensuite mis dans une pile, qui est lu lors du rendu de la scène.")
 'next
 (para "Nos objets deviennent persistants donc quand on a fini de les utiliser, il faut les enlever en faisant:")
 (code (clear))
 'next
 (para "Puis créons une sphere persistante:")
 (code (build-sphere 10 10))
 (it "build-sphere prend en paramètre la compléxité voulu de l'objet."))

; chapitre 2: une premiere animation
(chapitrage2 'animation)

(slide
 #:title "La machine à états"
 (para/center "Il est important de comprendre le fonctionnement des \"états\" dans Fluxus. Voyons le code suivant:")
 'next
 (para/center
  (code (clear)
        (colour (vector 1 0 0))
        (build-cube)))
 (para/center "On vient de créer un cube rouge. La fonction colour, tout comme les fonctions rotate, translate, scale et autres prennent en paramètre un triplet sous forme de tableau (un vecteur en Scheme).")
 'next
 (it "Pour colour, ces valeurs représentent le système de couleurs RVB (rouge vert bleu).")
 (it "Pour les autres, ce sont les coordonnées dans l'espace suivant x, y et z."))

(slide
 #:title "La machine à états"
 (para "Allons encore plus loin:")
 (code (clear)
       (colour (vector 1 0 0))
       (build-cube)
       (translate (vector 1.1 0 0))
       (colour (vector 0 1 0))
       (build-cube))
 (para "Ici nous avons crée un cube rouge et à coté un cube vert."))

(slide
 #:title "La machine à états"
 (para "Nous allons voir le fonctionnement de la commande with-state. En Scheme on utiliserait:")
 (blank)
 (code (local [(define ...)]
         ...))
 'next
 (para "Le principe est le même, ici on cherche à isoler un état du reste du script.Pour mieux s'en rendre compte reprenons l'exemple précédent !"))

(slide
 #:title "La machine à états"
 (code (clear)
       (colour (vector 1 0 0))
       (with-state
        (colour (vector 0 1 0))
        (build-cube))
       (translate (vector 1.1 0 0))
       (build-cube)))

(slide
 #:title "La machine à états"
 (para/center "Mais que s'est-il passé ?")
 'next
 (para "Nous avions dabord défini une couleur, mais nous avons ensuite crée un contexte, un nouvel état avec une couleur propre, puis nous avons dessiné un cube de cette couleur.")
 'next
 (para "sorti de ce contexte, nous retournons au contexte ou la couleur était rouge, d'ou la couleur du deuxième cube."))

(slide
 #:title "Les animations"
 (para/center "Avec tout ce que l'on vient de voir, on peut désormais aborder les animations.")
 'next
 (para/center "Dabord créons un objet, donnons lui une couleur, une forme et surtout un nom:")
 'next
 (code (clear)
       (define monCube
         (with-state
          (colour (vector 0 0 1))
          (build-cube)))))

(slide
 #:title "Les animations"
 (para/center "On va maintenant créer la fonction qui va faire tourner de quelques degrés notre objet:")
 'next
 (code (define (animation)
         (with-primitive monCube
                         (rotate (vector 2 0 0)))))
 (blank)
 'next
 (para/center "Et enfin every-frame qui va appeler notre fonction animation à chaque fois qu'on redessine l'écran:")
 'next
 (code (every-frame (animation))))

(slide
 #:title "Les animations"
 #:layout 'top
 (para/center "Notre cube se met alors à tourner sur lui même, la fonction with-primitive est très utile, elle permet de cibler un objet et de modifier son contexte.")
 'next
 (para/center "Un autre exemple plus compliqué:")
 (scale
  (code
   (define monCube
     (with-state
      (colour (vector 1 0 0))
      (build-cube)))
   (define monDonut
     (with-state
      (colour (vector 0 0 1))
      (scale (vector 2 2 2))
      (build-torus 0.5 1 12 12)))
   (define (animation)
     (with-primitive monCube
                     (rotate (vector 1 0 0)))
     (with-primitive monDonut
                     (rotate (vector 0 2 0))))
   (every-frame (animation))) 0.5))

(slide
 #:title "Les animations"
 #:layout 'top
 (para/center "Encore une fois, il est important de distinguer les deux façons de dessiner à l'écran:")
 'next
 (scale
  (code
   (define angleCube 0)
   (define angleDonut 0)
   (define (animation)
     (with-state
      (colour (vector 1 0 0))
      (set! angleCube (+ angleCube 1))
      (rotate (vector angle1 0 0))
      (draw-cube))
     (with-state
      (colour (vector 0 0 1))
      (set! angleDonut (+ angleDonut 2))
      (rotate (vector 0 angleDonut 0))
      (draw-torus)))
   (every-frame (animation))) 0.75)
 'next
 (para/center "rend exactement la même chose !"))

; chapitre 3: la récursivité
(chapitrage2 'recursiv)
(slide
 #:title "La récursivité"
 (para/center "Nous allons maintenant voir des exemples de programmation récursive pour l'affichage de figures un peu plus complexes:")
 'next
 (scale
  (code
   (clear)
   (define (rangéeDeCubes n)
     (cond
       ((not (zero? n))
        (draw-cube)
        (rangéeDeCubes (- n 1)))))
   (every-frame (rangéeDeCubes 10))) 0.75)
 (blank)
 'next
 (para/center "On dessine récursivement une rangée de 10 cubes, c'est une manière très puissante de dessiner, de plus le code n'est pas si long que ça !"))

(slide
 #:title "La récursivité"
 (para/center "Voici un autre exemple plus complexe, on va dessiner plusieurs cubes imbriqués les uns dans les autres:")
 'next
 (scale
  (code
   (clear)
   (hint-none) \; on masque les faces des objets
   (hint-wire) \; on affiche les arrêtes des objets
   (backfacecull 0) \; on affiche les arrêtes non apparentes
   (hint-ignore-depth) \; on désactive l\'effet de profondeur
   (hint-anti-alias) \; on active l\'anticrénelage
   
   (define (cubesEmboités n)
     (if (zero? n)
         0
         (begin
           (with-state
            (wire-colour (vector 0 1 0))
            (scale (vector n n n))
            (draw-cube))
           (cubesEmboités (- n 1)))))
   
   (every-frame (cubesEmboités 10))) 0.5)
 'next
 (para
  (it "Remarquez que if et cond fonctionnent comme dans DrScheme")
  (it "vous pouvez imaginer des scripts beaucoup plus complexes avec ça !")))


; chapitre 4: l'audio

(chapitrage2 'audio)

(slide
 #:title "L'audio avec Fluxus"
 (para/center "Le principal interêt de Fluxus est l'interaction de sons directement dans vos animations.")
 'next
 (para/center "Il faudra cependant installer un serveur Jack qui servira à \"router\" un flux audio vers Fluxus.")
 'next
 (para/center "Nous verrons comment utiliser le son dans Fluxus à partir de GNU/Linux et de Mac OS 10.5"))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Sous Ubuntu nous utiliserons qjackctl, une interface graphique au serveur Jack qui nous évitera de passer par le terminal.")
 (scale (image "ressources/interfaceqjackctl.png") 1.5)
 'next
 (para/center "Lancez qjackctl puis allez dans Setup."))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Modifiez la valeur du buffer audio (Frames/Period) en mettant 2048:")
 (scale (image "ressources/setupqjackctl.png") 1.5)
 'next
  (para
  (it "plus le buffer est élevé, moins vous aurrez")
  (it "de chances que votre source audio ne s'entrecoupe...")))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Il est évidement possible d'utiliser Jack en temps réel, pour cela il faudrait recompiler le noyau Linux avec les bonnes options et reconfigurer Jack par la suite mais celà sort du contexte de Fluxus."))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Nous allons utiliser le lecteur audio Amarok qui nous permettra en le modifiant de lui indiquer d'envoyer son flux audio vers Jack.")
 'next
 (para/center "Installons Amarok via apt-get:")
 'next
 (cmd ">sudo apt-get install amarok")
 'next
 (para/center "Amarok utilise le moteur Xine pour décoder l'audio, malheureusement la version bundlée avec Ubuntu ne permet pas de rediriger le flux audio vers un serveur Jack.")
 'next
 (para/center "Nous allons y remedier en compilant notre propre version des librairies de Xine."))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "On récupère le code source de xine-lib sur leur site:")
 'next
 (cmd "http://www.xine-project.org/")
 'next
 (para/center "Téléchargez la puis décompressez la dans notre répertoire sources d'installation.")
 'next
 (para/center "On compile et on l'installe:")
 'next
 (para
  (cmd ">./configure")
  (cmd ">make")
  (cmd ">sudo make install")))

(slide
 #:title "L'audio sous GNU/Linux"
 (image "ressources/logoamarok.png")
 (blank)
 (blank)
 (para/center "Maintenant on lance Amarok, puis rendez vous dans Configuration puis Configurer Amarok."))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Rendez vous dans Moteur, puis mettez le module de sortie de Xine sur Jack.")
 (scale (image "ressources/configamarok.png") 1.5))

(slide
 #:title "L'audio sous GNU/linux"
 (para/center "Validez puis rendez vous sur l'interface de qjackctl.")
 'next
 (para/center "Lancez le serveur Jackd (boutton Lancer).")
 'next
 (para/center "Dans Amarok lancez une playlist, si vous n'entendez rien c'est que Jack n'as pas redirigé le flux venant de Xine vers vos haut-parleurs."))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Dans l'interface de qjackctl ouvrez le menu de connexions (boutton Connect), on va router la sortie de Xine vers les haut-parleurs (dans System).")
 (scale (image "ressources/connecjack.png") 1.5))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Dans Fluxus, on va déclarer un port d'écoute pour que Jack puisse s'y connecter.")
 'next
 (para/center "Dans le TOPLEVEL de Fluxus:")
 'next
 (code (start-audio "" 2048 44100))
 (para
  (it "44100 est la fréquence d'échantillonnage")
  (it "utilisée par Jack pour le réencodage des flux audio,")
  (it "par défaut la valeur est bonne, on ne va donc pas la toucher.")))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Fluxus devrait apparaître dans la fenêtre des connexions dans qjackctl, on peut maintenant relier Xine à Fluxus.")
 'next
 (para/center "Maintenant dans un espace de travail de Fluxus, on va créer un cube qui changera de couleur en fonction de la musique:")
 (code (clear)
       (define (cube-animé)
         (with-state
          (colour (vector (gh 0) (gh 1) (gh 2)))
          (draw-cube)))
       (every-frame (cube-animé))))

(slide
 #:title "L'audio sous GNU/Linux"
 (para/center "Si on lance le script, on voit le cube changer de couleur !")
 'next
 (para/center "On a utilisé la commande gh qui, en fonction du numéro d'harmonique qu'on lui donne, nous renvoie un réel.")
 'next
 (it "Il y a 16 harmoniques en tout..."))

(slide
 #:title "L'audio sous Mac OS 10.5"
 (para/center "Tout comme sous Ubuntu, il faudra installer une interface graphique a Jack.")
 'next
 (para/center "Nous utiliserons JackOSX qui est disponible à cette adresse:")
 'next
 (cmd "http://www.jackosx.com/")
 'next
 (para/center "Installé, il faudra le lancer et se rendre dans la barre de menu de JackPilot puis sélectionner \"Configuration audio et MIDI\":")
 (scale (image "ressources/jackosx1.png") 1.5))

(slide
 #:title "L'audio sous Mac OS 10.5"
 (para/center "On va créer une interface virtuelle qui comprendra:")
 'next
 (item "en entrées:")
 (subitem "le micro de l'ordinateur")
 (subitem "l'entrée auxiliaire \"line-in\"")
 'next
 (item "en sorties:")
 (subitem "les haut-parleurs de l'ordinateur")
 (subitem "une sortie auxiliaire si disponible (prise casque par exemple)")
 'next
 (para
  (it "cette interface virtuelle est appelée")
  (it "un périphérique agrégé.")))

(slide
 #:title "L'audio sous Mac OS 10.5"
 (para/center "Dans la barre de menu de l'outil \"Configuration audio et MIDI\" sélectionnez Audio puis \"Ouvrir l'éditeur de périphériques audio\" (raccourci clavier MAJ-POMME-A)")
 'next
 (para/center "Appuyez sur le + pour créer notre périphérique agrégé. N'oubliez pas de cocher tout les périphériques qui nous intéressent puis fermez la fenêtre.")
 'next
 (para/center "Dans la fenêtre de configuration audio et MIDI, selectionnez le périphérique agrégé comme entrée et sortie par défaut.")
 'next
 (para/center "Maintenant il ne nous reste qu'à configurer Jack pour qu'il utilise notre périphérique agrégé."))

(slide
 #:title "L'audio sous Mac OS 10.5"
 (para/center "Dans la barre des menus de JackPilot, allez dans les préférences (raccourci clavier POMME-,) puis sélectionnez votre périphérique agrégé comme interface. Vérifiez que le buffer audio est bien sur 2048.")
 'next
 (para/center "Fermez la fenêtre des préférences, et lançons iTunes.")
 'next
 (para/center "Après avoir lancé un morceau, retournez dans qjackctl, iTunes devrait apparaître dans la colonne de gauche."))

(slide
 #:title "L'audio sous Mac OS 10.5"
 (para/center "Referez vous à l'audio sous GNU/Linux pour savoir comment ouvrir un port Jack dans Fluxus. Il devrait ensuite être detecté dans JackPilot.")
 (scale (image "ressources/connectjackpanel.png") 1.5)
 'next
 (para
  (it "Attention, il faudra lancer le serveur Jack avant de lancer un morceau")
  (it "sinon iTunes ou Fluxus pourrait planter.")))

(slide
 #:title "L'audio sous Mac OS 10.5"
 (para/center "Vous pourrez essayer le script vu dans la partie audio sous GNU/Linux pour voir si le son est bien detecté.")
 'next
 (para/center "Jack est un outil très utile mais aussi très vaste qui mériterait un slideshow à lui tout seul.")
 'next
 (para/center "Il permet de prendre théoriquement n'importe quel flux audio et de le passer à une autre application. Sous Mac OS X, on peut notament prendre l'audio de Firefox ou VLC et le rediriger vers Garageband par exemple !"))

; chapitre 5: DrScheme et Fluxus

(chapitrage2 'drscheme)

(slide
 #:title "Fluxus et DrScheme"
 (para/center "Pour rappel DrScheme est l'environnement de developpement intégré à PLT Scheme.")
 'next
 (para/center "Il est possible de l'utiliser à la place de l'éditeur de Fluxus.")
 'next
 (para/center "On pert évidement l'interêt du livecoding mais pour gérer de gros scripts il est préférable de passer par DrScheme."))

(slide
 #:title "Fluxus et DrScheme"
 (para/center "DrScheme propose:")
 'next
 (item "un debuggeur.")
 'next
 (item "la coloration syntaxique.")
 'next
 (item "l'indentation automatique.")
 'next
 (para/center "Pour cela il faut donc utiliser Fluxus comme module."))

(slide
 #:title "Fluxus et DrScheme"
 (para/center "Il faudra s'assurer que fluxus est bien dans votre répertoire des modules de PLT Scheme. Nous allons en fait faire un lien symbolique dans le répertoire des modules de PLT Scheme qui pointera vers le répertoire de Fluxus:")
 'next
 (para/center "Sous GNU/Linux, ouvrons un terminal:")
 'next
 (cmd-sized 18 ">sudo ln -s /usr/local/bin/fluxus /usr/local/lib/plt/collects/fluxus-016"))

(slide
 #:title "Fluxus et DrScheme"
 (para/center "Sous Mac OS 10.5, dans un terminal:")
 'next
 (cmd-sized 16 ">sudo ln -s /opt/local/lib/fluxus-016 /Applications/PLT\\ Scheme\\ v4.1.x/collects/fluxus-016")
 'next
 (it "x étant la version de PLT Scheme que vous avez installé")
 'next
 (para/center "Ainsi dans DrScheme, vous n'aurrez qu'a rajouter cette ligne en début de votre script:")
 (code (require fluxus-016/drflux)))


; chapitre 6: Les Fluxus-like

(chapitrage2 'fluxus-like)

(slide
 #:title "Les Fluxus-like"
 #:layout 'top
 (para/center "Il existe d'autres applications comme Fluxus, et la plupart ont évidement choisi Lisp ou Scheme comme langage de programmation car ils sont très appropriés à ce style d'utilisation.")
 'next
 (item "Impromptu")
 (subitem "Seulement sur Mac OS X, il permet ce que Fluxus propose déjà mais avec une interface plus \"intuitive\"")
 (image "ressources/impromptu.jpg"))

(slide
 #:title "Les Fluxus-like"
 (item "SuperCollider")
 (subitem "multiplateforme, il possède son propre langage de programmation orienté objet, il se spécialise surtout dans l'audio.")
 'next
 (item "Livecoding Python")
 (subitem "une librairie qui se met entre le programmeur Python et l'API OpenGl. Elle permet le livecoding et l'évaluation à la volée du code."))


; chapitr 7: Fluxus en constante évolution

(chapitrage2 'wip)

(slide
 #:title "Le futur de Fluxus"
 (para/center "Fluxus est un projet en constante évolution, il dispose d'une mailing-list très active. Actuellement l'unification et le bundling des dépendances est en cours, ce qui permettra une installation plus aisée, et pourquoi pas un portage sous Windows éventuellement. Aussi une installation moins hasardeuse est prévue pour que tout les modules comme le moteur physique intégré Ode ou encore le module de création de jeux vidéo Frisbee fonctionnent \"out of the box\"."))

(slide
 #:title "Le futur de Fluxus"
 (para/center "Depuis peu, il permet à l'aide du serveur Jack de créer de la musique récursivement, c'est encore très expérimental et les résultats sont assez aléatoire mais on remarque que les possibilités apportées par la souplesse du langage Scheme sont quasi infinies."))

(slide
 (code (the end)))